@@ -1,7 +1,7 @@
 from enum import Enum
 
 
-class RPCMessageType(Enum):
+class RPCMessageType(str, Enum):
     STATUS = 'status'
     WARNING = 'warning'
     STARTUP = 'startup'
@@ -17,8 +17,21 @@ class RPCMessageType(Enum):
     PROTECTION_TRIGGER = 'protection_trigger'
     PROTECTION_TRIGGER_GLOBAL = 'protection_trigger_global'
 
+    STRATEGY_MSG = 'strategy_msg'
+
+    WHITELIST = 'whitelist'
+    ANALYZED_DF = 'analyzed_df'
+
     def __repr__(self):
         return self.value
 
     def __str__(self):
         return self.value
+
+
+# Enum for parsing requests from ws consumers
+class RPCRequestType(str, Enum):
+    SUBSCRIBE = 'subscribe'
+
+    WHITELIST = 'whitelist'
+    ANALYZED_DF = 'analyzed_df'
