@@ -4,9 +4,9 @@ This module defines a base class for auto-hyperoptable strategies.
 """
 import logging
 from pathlib import Path
-from typing import Any, Dict, Iterator, List, Tuple
+from typing import Any, Dict, Iterator, List, Tuple, Type, Union
 
-from freqtrade.enums import RunMode
+from freqtrade.constants import Config
 from freqtrade.exceptions import OperationalException
 from freqtrade.misc import deep_merge_dicts, json_load
 from freqtrade.optimize.hyperopt_tools import HyperoptTools
@@ -22,7 +22,7 @@ class HyperStrategyMixin:
      strategy logic.
     """
 
-    def __init__(self, config: Dict[str, Any], *args, **kwargs):
+    def __init__(self, config: Config, *args, **kwargs):
         """
         Initialize hyperoptable strategy mixin.
         """
@@ -31,7 +31,10 @@ class HyperStrategyMixin:
         self.ft_sell_params: List[BaseParameter] = []
         self.ft_protection_params: List[BaseParameter] = []
 
-        self._load_hyper_params(config.get('runmode') == RunMode.HYPEROPT)
+        params = self.load_params_from_file()
+        params = params.get('params', {})
+        self._ft_params_from_file = params
+        # Init/loading of parameters is done as part of ft_bot_start().
 
     def enumerate_parameters(self, category: str = None) -> Iterator[Tuple[str, BaseParameter]]:
         """
@@ -51,28 +54,13 @@ class HyperStrategyMixin:
         for par in params:
             yield par.name, par
 
-    @classmethod
-    def detect_parameters(cls, category: str) -> Iterator[Tuple[str, BaseParameter]]:
-        """ Detect all parameters for 'category' """
-        for attr_name in dir(cls):
-            if not attr_name.startswith('__'):  # Ignore internals, not strictly necessary.
-                attr = getattr(cls, attr_name)
-                if issubclass(attr.__class__, BaseParameter):
-                    if (attr_name.startswith(category + '_')
-                            and attr.category is not None and attr.category != category):
-                        raise OperationalException(
-                            f'Inconclusive parameter name {attr_name}, category: {attr.category}.')
-                    if (category == attr.category or
-                            (attr_name.startswith(category + '_') and attr.category is None)):
-                        yield attr_name, attr
-
     @classmethod
     def detect_all_parameters(cls) -> Dict:
         """ Detect all parameters and return them as a list"""
         params: Dict[str, Any] = {
-            'buy': list(cls.detect_parameters('buy')),
-            'sell': list(cls.detect_parameters('sell')),
-            'protection': list(cls.detect_parameters('protection')),
+            'buy': list(detect_parameters(cls, 'buy')),
+            'sell': list(detect_parameters(cls, 'sell')),
+            'protection': list(detect_parameters(cls, 'protection')),
         }
         params.update({
             'count': len(params['buy'] + params['sell'] + params['protection'])
@@ -80,21 +68,49 @@ class HyperStrategyMixin:
 
         return params
 
-    def _load_hyper_params(self, hyperopt: bool = False) -> None:
+    def ft_load_params_from_file(self) -> None:
+        """
+        Load Parameters from parameter file
+        Should/must run before config values are loaded in strategy_resolver.
+        """
+        if self._ft_params_from_file:
+            # Set parameters from Hyperopt results file
+            params = self._ft_params_from_file
+            self.minimal_roi = params.get('roi', getattr(self, 'minimal_roi', {}))
+
+            self.stoploss = params.get('stoploss', {}).get(
+                'stoploss', getattr(self, 'stoploss', -0.1))
+            trailing = params.get('trailing', {})
+            self.trailing_stop = trailing.get(
+                'trailing_stop', getattr(self, 'trailing_stop', False))
+            self.trailing_stop_positive = trailing.get(
+                'trailing_stop_positive', getattr(self, 'trailing_stop_positive', None))
+            self.trailing_stop_positive_offset = trailing.get(
+                'trailing_stop_positive_offset',
+                getattr(self, 'trailing_stop_positive_offset', 0))
+            self.trailing_only_offset_is_reached = trailing.get(
+                'trailing_only_offset_is_reached',
+                getattr(self, 'trailing_only_offset_is_reached', 0.0))
+
+    def ft_load_hyper_params(self, hyperopt: bool = False) -> None:
         """
         Load Hyperoptable parameters
+        Prevalence:
+        * Parameters from parameter file
+        * Parameters defined in parameters objects (buy_params, sell_params, ...)
+        * Parameter defaults
         """
-        params = self.load_params_from_file()
-        params = params.get('params', {})
-        self._ft_params_from_file = params
-        buy_params = deep_merge_dicts(params.get('buy', {}), getattr(self, 'buy_params', {}))
-        sell_params = deep_merge_dicts(params.get('sell', {}), getattr(self, 'sell_params', {}))
-        protection_params = deep_merge_dicts(params.get('protection', {}),
+
+        buy_params = deep_merge_dicts(self._ft_params_from_file.get('buy', {}),
+                                      getattr(self, 'buy_params', {}))
+        sell_params = deep_merge_dicts(self._ft_params_from_file.get('sell', {}),
+                                       getattr(self, 'sell_params', {}))
+        protection_params = deep_merge_dicts(self._ft_params_from_file.get('protection', {}),
                                              getattr(self, 'protection_params', {}))
 
-        self._load_params(buy_params, 'buy', hyperopt)
-        self._load_params(sell_params, 'sell', hyperopt)
-        self._load_params(protection_params, 'protection', hyperopt)
+        self._ft_load_params(buy_params, 'buy', hyperopt)
+        self._ft_load_params(sell_params, 'sell', hyperopt)
+        self._ft_load_params(protection_params, 'protection', hyperopt)
 
     def load_params_from_file(self) -> Dict:
         filename_str = getattr(self, '__file__', '')
@@ -117,7 +133,7 @@ class HyperStrategyMixin:
 
         return {}
 
-    def _load_params(self, params: Dict, space: str, hyperopt: bool = False) -> None:
+    def _ft_load_params(self, params: Dict, space: str, hyperopt: bool = False) -> None:
         """
         Set optimizable parameter values.
         :param params: Dictionary with new parameter values.
@@ -126,7 +142,7 @@ class HyperStrategyMixin:
             logger.info(f"No params for {space} found, using default values.")
         param_container: List[BaseParameter] = getattr(self, f"ft_{space}_params")
 
-        for attr_name, attr in self.detect_parameters(space):
+        for attr_name, attr in detect_parameters(self, space):
             attr.name = attr_name
             attr.in_space = hyperopt and HyperoptTools.has_space(self.config, space)
             if not attr.category:
@@ -157,3 +173,26 @@ class HyperStrategyMixin:
             if not p.optimize or not p.in_space:
                 params[p.category][name] = p.value
         return params
+
+
+def detect_parameters(
+        obj: Union[HyperStrategyMixin, Type[HyperStrategyMixin]],
+        category: str
+        ) -> Iterator[Tuple[str, BaseParameter]]:
+    """
+    Detect all parameters for 'category' for "obj"
+    :param obj: Strategy object or class
+    :param category: category - usually `'buy', 'sell', 'protection',...
+    """
+    for attr_name in dir(obj):
+        if not attr_name.startswith('__'):  # Ignore internals, not strictly necessary.
+            attr = getattr(obj, attr_name)
+            if issubclass(attr.__class__, BaseParameter):
+                if (attr_name.startswith(category + '_')
+                        and attr.category is not None and attr.category != category):
+                    raise OperationalException(
+                        f'Inconclusive parameter name {attr_name}, category: {attr.category}.')
+
+                if (category == attr.category or
+                        (attr_name.startswith(category + '_') and attr.category is None)):
+                    yield attr_name, attr
